@@ -12,6 +12,7 @@
 #include <peer/peermanager.h>
 #include <torrent/job.h>
 #include <torrent/torrent.h>
+#include <util/error.h>
 #include <util/fileops.h>
 #include <util/functions.h>
 #include <util/log.h>
@@ -58,7 +59,7 @@ void Cache::changeTmpDir(const QString &ndir)
 
 bool Cache::mappedModeAllowed()
 {
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
     return MaxOpenFiles() - bt::PeerManager::connectionLimits().totalConnections() > 100;
 #else
     return true; // there isn't a file handle limit on windows
