@@ -7,16 +7,9 @@
 #include "socket.h"
 #include <QtGlobal>
 
-#include <arpa/inet.h>
 #include <errno.h>
-#include <netdb.h>
-#include <netinet/in.h>
-#include <netinet/tcp.h>
 #include <string.h>
-#include <sys/ioctl.h>
-#include <sys/socket.h>
 #include <sys/types.h>
-#include <unistd.h>
 
 #if defined(Q_OS_LINUX) && !defined(__FreeBSD_kernel__)
 #include <asm/ioctls.h>
@@ -34,13 +27,24 @@
 
 #include <util/log.h>
 
-#ifdef Q_WS_WIN
+#ifndef Q_OS_WIN
+#include <arpa/inet.h>
+#include <netdb.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>
+#include <sys/ioctl.h>
+#include <sys/socket.h>
+#include <unistd.h>
+#else
 #include <util/win32.h>
+#include <ws2tcpip.h>
 #define SHUT_RDWR SD_BOTH
 #undef errno
 #define errno WSAGetLastError()
 #endif
+
 using namespace bt;
+using namespace Qt::Literals::StringLiterals;
 
 namespace net
 {
@@ -80,6 +84,21 @@ Socket::Socket(bool tcp, int ip_version)
         Out(SYS_GEN | LOG_IMPORTANT) << QStringLiteral("Cannot create socket : %1").arg(QString::fromUtf8(strerror(errno))) << endl;
     m_fd = fd;
 
+    // Try using dual-stack for IPv6
+    if (m_fd != -1 && m_ip_version == 6) {
+        const int no = 0;
+#ifndef Q_OS_WIN
+        int err = setsockopt(m_fd, IPPROTO_IPV6, IPV6_V6ONLY, &no, sizeof(no));
+#else
+        int err = setsockopt(m_fd, IPPROTO_IPV6, IPV6_V6ONLY, reinterpret_cast<const char *>(&no), sizeof(no));
+#endif
+        if (err < 0) {
+            Out(SYS_GEN | LOG_IMPORTANT) << u"Failed to set dual-stack option for IPv6 socket : %1"_s.arg(QString::fromUtf8(strerror(errno))) << endl;
+        } else {
+            dualstack = true;
+        }
+    }
+
 #if defined(Q_OS_MACX) || defined(Q_OS_DARWIN)
     int val = 1;
     if (setsockopt(m_fd, SOL_SOCKET, SO_NOSIGPIPE, &val, sizeof(int)) < 0) {
@@ -92,7 +111,7 @@ Socket::~Socket()
 {
     if (m_fd >= 0) {
         shutdown(m_fd, SHUT_RDWR);
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN
         ::closesocket(m_fd);
 #else
         ::close(m_fd);
@@ -121,7 +140,7 @@ void Socket::close()
 {
     if (m_fd >= 0) {
         shutdown(m_fd, SHUT_RDWR);
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN
         ::closesocket(m_fd);
 #else
         ::close(m_fd);
@@ -133,14 +152,26 @@ void Socket::close()
 
 void Socket::setBlocking(bool on)
 {
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
     int flag = fcntl(m_fd, F_GETFL, 0);
     if (!on)
         fcntl(m_fd, F_SETFL, flag | O_NONBLOCK);
     else
         fcntl(m_fd, F_SETFL, flag & ~O_NONBLOCK);
 #else
-    u_long b = on ? 1 : 0;
+    /** This makes no sense...
+
+        > FIONBIO
+        > -------
+        > The lpvInBuffer parameter points at an unsigned long (QoS), which is
+        > nonzero if non-blocking mode is to be enabled and zero if it is to be
+        > disabled.
+
+        https://learn.microsoft.com/en-us/windows/win32/winsock/winsock-ioctls#fionbio
+
+        But here we have to pass a pointer to 0 to enable the non-blocking mode?
+    */
+    u_long b = on ? 0 : 1;
     ioctlsocket(m_fd, FIONBIO, &b);
 #endif
 }
@@ -149,9 +180,9 @@ bool Socket::connectTo(const Address &a)
 {
     int len = 0;
     struct sockaddr_storage ss;
-    a.toSocketAddress(&ss, len);
+    a.toSocketAddress(&ss, len, dualstack);
     if (::connect(m_fd, (struct sockaddr *)&ss, len) < 0) {
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
         if (errno == EINPROGRESS)
 #else
         if (errno == WSAEINVAL || errno == WSAEALREADY || errno == WSAEWOULDBLOCK)
@@ -178,7 +209,7 @@ bool Socket::bind(const QString &ip, Uint16 port, bool also_listen)
 bool Socket::bind(const net::Address &addr, bool also_listen)
 {
     int val = 1;
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
     if (setsockopt(m_fd, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(int)) < 0)
 #else
     if (setsockopt(m_fd, SOL_SOCKET, SO_REUSEADDR, (char *)&val, sizeof(int)) < 0)
@@ -208,7 +239,7 @@ bool Socket::bind(const net::Address &addr, bool also_listen)
 
 int Socket::send(const bt::Uint8 *buf, int len)
 {
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
     int ret = ::send(m_fd, buf, len, MSG_NOSIGNAL);
 #else
     int ret = ::send(m_fd, (char *)buf, len, MSG_NOSIGNAL);
@@ -225,7 +256,7 @@ int Socket::send(const bt::Uint8 *buf, int len)
 
 int Socket::recv(bt::Uint8 *buf, int max_len)
 {
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
     int ret = ::recv(m_fd, buf, max_len, 0);
 #else
     int ret = ::recv(m_fd, (char *)buf, max_len, 0);
@@ -250,7 +281,7 @@ int Socket::sendTo(const bt::Uint8 *buf, int len, const Address &a)
 {
     int alen = 0;
     struct sockaddr_storage ss;
-    a.toSocketAddress(&ss, alen);
+    a.toSocketAddress(&ss, alen, dualstack);
     int ret = ::sendto(m_fd, (char *)buf, len, 0, (struct sockaddr *)&ss, alen);
     if (ret < 0) {
         if (errno == EAGAIN || errno == EWOULDBLOCK)
@@ -267,7 +298,7 @@ int Socket::recvFrom(bt::Uint8 *buf, int max_len, Address &a)
 {
     struct sockaddr_storage ss;
     socklen_t slen = sizeof(ss);
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
     int ret = ::recvfrom(m_fd, buf, max_len, 0, (struct sockaddr *)&ss, &slen);
 #else
     int ret = ::recvfrom(m_fd, (char *)buf, max_len, 0, (struct sockaddr *)&ss, &slen);
@@ -308,7 +339,7 @@ bool Socket::setTOS(unsigned char type_of_service)
 #else
         unsigned char c = type_of_service;
 #endif
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
         if (setsockopt(m_fd, IPPROTO_IP, IP_TOS, &c, sizeof(c)) < 0)
 #else
         if (setsockopt(m_fd, IPPROTO_IP, IP_TOS, (char *)&c, sizeof(c)) < 0)
@@ -320,7 +351,11 @@ bool Socket::setTOS(unsigned char type_of_service)
         }
     } else {
 #if defined(IPV6_TCLASS)
+#ifndef Q_OS_WIN
         int c = type_of_service;
+#else
+        char c = type_of_service;
+#endif
         if (setsockopt(m_fd, IPPROTO_IPV6, IPV6_TCLASS, &c, sizeof(c)) < 0) {
             Out(SYS_CON | LOG_NOTICE)
                 << QStringLiteral("Failed to set traffic class to %1 : %2").arg((int)type_of_service).arg(QString::fromUtf8(strerror(errno))) << endl;
@@ -335,7 +370,7 @@ bool Socket::setTOS(unsigned char type_of_service)
 Uint32 Socket::bytesAvailable() const
 {
     int ret = 0;
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
     if (ioctl(m_fd, FIONREAD, &ret) < 0)
 #else
     if (ioctlsocket(m_fd, FIONREAD, (u_long *)&ret) < 0)
@@ -347,12 +382,12 @@ Uint32 Socket::bytesAvailable() const
 
 bool Socket::connectSuccesFull()
 {
-    if (m_state != CONNECTING)
+    if (m_state != CONNECTING && m_state != CONNECTED)
         return false;
 
     int err = 0;
     socklen_t len = sizeof(int);
-#ifndef Q_WS_WIN
+#ifndef Q_OS_WIN
     if (getsockopt(m_fd, SOL_SOCKET, SO_ERROR, &err, &len) < 0)
 #else
     if (getsockopt(m_fd, SOL_SOCKET, SO_ERROR, (char *)&err, &len) < 0)
